# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class BillRunSchedule(BaseModel):
    """
    Container for information about the scheduled bill run. 
    """ # noqa: E501
    repeat_from: Optional[date] = Field(default=None, description="The start date of the scheduled bill run. ", alias="repeatFrom")
    repeat_to: Optional[date] = Field(default=None, description="The end date of of the scheduled bill run. ", alias="repeatTo")
    repeat_type: Optional[StrictStr] = Field(default=None, description="The repeat type of the bill run. ", alias="repeatType")
    run_time: Optional[StrictInt] = Field(default=None, description="The scheduled run time (hour) of day.  **Values:** 0 - 23 ", alias="runTime")
    weekly_on_day: Optional[List[StrictStr]] = Field(default=None, description="The repeat day in a week. ", alias="weeklyOnDay")
    monthly_on_end_of_month: Optional[StrictBool] = Field(default=None, description="Whether to schedule monthly bill run on the end of month or the specific day of month. This field is available only when repeatType is set to monthly and repeatFrom is set to the end of month.  For example: - When repeatFrom = '2024-04-30' and monthlyOnEndOfMonth = true, next bill run will be scheduled on 2024-05-31 - When repeatFrom = '2024-04-30' and monthlyOnEndOfMonth = false, next bill run will be scheduled on 2024-05-30 ", alias="monthlyOnEndOfMonth")
    __properties: ClassVar[List[str]] = ["repeatFrom", "repeatTo", "repeatType", "runTime", "weeklyOnDay", "monthlyOnEndOfMonth"]

    @field_validator('repeat_type')
    def repeat_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['None', 'Daily', 'Weekly', 'Monthly']):
            raise ValueError("must be one of enum values ('None', 'Daily', 'Weekly', 'Monthly')")
        return value

    @field_validator('weekly_on_day')
    def weekly_on_day_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        for i in value:
            if i not in set(['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']):
                raise ValueError("each list item must be one of ('Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of BillRunSchedule from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of BillRunSchedule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "repeatFrom": obj.get("repeatFrom"),
            "repeatTo": obj.get("repeatTo"),
            "repeatType": obj.get("repeatType"),
            "runTime": obj.get("runTime"),
            "weeklyOnDay": obj.get("weeklyOnDay"),
            "monthlyOnEndOfMonth": obj.get("monthlyOnEndOfMonth")
        })
        return _obj


