# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from zuora_sdk.models.create_subscription_rate_plan import CreateSubscriptionRatePlan
from typing import Optional, Set
from typing_extensions import Self

class CreateAccountSubscription(BaseModel):
    """
    Container for subscription information, used if creating a subscription for the new account at the time of account creation. 
    """ # noqa: E501
    cpq_bundle_json_id__qt: Optional[Annotated[str, Field(strict=True, max_length=32)]] = Field(default=None, description="The Bundle product structures from Zuora Quotes if you utilize Bundling in Salesforce. Do not change the value in this field. ", alias="CpqBundleJsonId__QT")
    opportunity_close_date__qt: Optional[date] = Field(default=None, description="The closing date of the Opportunity. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ", alias="OpportunityCloseDate__QT")
    opportunity_name__qt: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The unique identifier of the Opportunity. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ", alias="OpportunityName__QT")
    quote_business_type__qt: Optional[Annotated[str, Field(strict=True, max_length=32)]] = Field(default=None, description="The specific identifier for the type of business transaction the Quote represents such as New, Upsell, Downsell, Renewal or Churn. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ", alias="QuoteBusinessType__QT")
    quote_number__qt: Optional[Annotated[str, Field(strict=True, max_length=32)]] = Field(default=None, description="The unique identifier of the Quote. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ", alias="QuoteNumber__QT")
    quote_type__qt: Optional[Annotated[str, Field(strict=True, max_length=32)]] = Field(default=None, description="The Quote type that represents the subscription lifecycle stage such as New, Amendment, Renew or Cancel. This field is used in Zuora data sources to report on Subscription metrics. If the subscription originated from Zuora Quotes, the value is populated with the value from Zuora Quotes. ", alias="QuoteType__QT")
    integration_id__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="ID of the corresponding object in NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="IntegrationId__NS")
    integration_status__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="Status of the subscription's synchronization with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="IntegrationStatus__NS")
    project__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The NetSuite project that the subscription was created from. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="Project__NS")
    sales_order__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The NetSuite sales order than the subscription was created from. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="SalesOrder__NS")
    sync_date__ns: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="Date when the subscription was synchronized with NetSuite. Only available if you have installed the [Zuora Connector for NetSuite](https://www.zuora.com/connect/app/?appId=265). ", alias="SyncDate__NS")
    auto_renew: Optional[StrictBool] = Field(default=None, description="If `true`, auto-renew is enabled. Default is `false`. ", alias="autoRenew")
    contract_effective_date: date = Field(description="Effective contract date for this subscription, as `yyyy-mm-dd`. ", alias="contractEffectiveDate")
    customer_acceptance_date: Optional[date] = Field(default=None, description="The date on which the services or products within a subscription have been accepted by the customer, as `yyyy-mm-dd`.  Default value is dependent on the value of other fields. See Notes section for more details. ", alias="customerAcceptanceDate")
    initial_term: Optional[StrictInt] = Field(default=None, description="Duration of the initial subscription term in whole months.  Default is 0.  ", alias="initialTerm")
    invoice_owner_account_key: Optional[StrictStr] = Field(default=None, description="Invoice owner account number or ID.  **Note:** This feature is in Limited Availability. If you wish to have access to the feature, submit a request at [Zuora Global Support](https://support.zuora.com).  ", alias="invoiceOwnerAccountKey")
    invoice_separately: Optional[StrictBool] = Field(default=None, description="Separates a single subscription from other subscriptions and invoices the charge independently.   If the value is `true`, the subscription is billed separately from other subscriptions. If the value is `false`, the subscription is included with other subscriptions in the account invoice. The default value is `false`.  Prerequisite: The default subscription setting `Enable Subscriptions to be Invoiced Separately` must be set to `Yes`. ", alias="invoiceSeparately")
    notes: Optional[StrictStr] = None
    renewal_term: Optional[StrictInt] = Field(default=None, description="Duration of the renewal term in whole months. Default is 0. ", alias="renewalTerm")
    service_activation_date: Optional[date] = Field(default=None, description="The date on which the services or products within a subscription have been activated and access has been provided to the customer, as `yyyy-mm-dd`.  Default value is dependent on the value of other fields. See Notes section for more details. ", alias="serviceActivationDate")
    subscribe_to_rate_plans: Optional[List[CreateSubscriptionRatePlan]] = Field(default=None, description="Container for one or more rate plans for this subscription. ", alias="subscribeToRatePlans")
    subscription_number: Optional[StrictStr] = Field(default=None, description="Subscription Number. The value can be up to 1000 characters.  If you do not specify a subscription number when creating a subscription for the new account, Zuora will generate a subscription number automatically.  If the account is created successfully, the subscription number is returned in the `subscriptionNumber` response field. ", alias="subscriptionNumber")
    term_start_date: Optional[date] = Field(default=None, description="The date on which the subscription term begins, as `yyyy-mm-dd`. If this is a renewal subscription, this date is different from the subscription start date. ", alias="termStartDate")
    term_type: StrictStr = Field(description="Possible values are: `TERMED`, `EVERGREEN`. ", alias="termType")
    __properties: ClassVar[List[str]] = ["CpqBundleJsonId__QT", "OpportunityCloseDate__QT", "OpportunityName__QT", "QuoteBusinessType__QT", "QuoteNumber__QT", "QuoteType__QT", "IntegrationId__NS", "IntegrationStatus__NS", "Project__NS", "SalesOrder__NS", "SyncDate__NS", "autoRenew", "contractEffectiveDate", "customerAcceptanceDate", "initialTerm", "invoiceOwnerAccountKey", "invoiceSeparately", "notes", "renewalTerm", "serviceActivationDate", "subscribeToRatePlans", "subscriptionNumber", "termStartDate", "termType"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateAccountSubscription from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in subscribe_to_rate_plans (list)
        _items = []
        if self.subscribe_to_rate_plans:
            for _item in self.subscribe_to_rate_plans:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subscribeToRatePlans'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateAccountSubscription from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "CpqBundleJsonId__QT": obj.get("CpqBundleJsonId__QT"),
            "OpportunityCloseDate__QT": obj.get("OpportunityCloseDate__QT"),
            "OpportunityName__QT": obj.get("OpportunityName__QT"),
            "QuoteBusinessType__QT": obj.get("QuoteBusinessType__QT"),
            "QuoteNumber__QT": obj.get("QuoteNumber__QT"),
            "QuoteType__QT": obj.get("QuoteType__QT"),
            "IntegrationId__NS": obj.get("IntegrationId__NS"),
            "IntegrationStatus__NS": obj.get("IntegrationStatus__NS"),
            "Project__NS": obj.get("Project__NS"),
            "SalesOrder__NS": obj.get("SalesOrder__NS"),
            "SyncDate__NS": obj.get("SyncDate__NS"),
            "autoRenew": obj.get("autoRenew"),
            "contractEffectiveDate": obj.get("contractEffectiveDate"),
            "customerAcceptanceDate": obj.get("customerAcceptanceDate"),
            "initialTerm": obj.get("initialTerm"),
            "invoiceOwnerAccountKey": obj.get("invoiceOwnerAccountKey"),
            "invoiceSeparately": obj.get("invoiceSeparately"),
            "notes": obj.get("notes"),
            "renewalTerm": obj.get("renewalTerm"),
            "serviceActivationDate": obj.get("serviceActivationDate"),
            "subscribeToRatePlans": [CreateSubscriptionRatePlan.from_dict(_item) for _item in obj["subscribeToRatePlans"]] if obj.get("subscribeToRatePlans") is not None else None,
            "subscriptionNumber": obj.get("subscriptionNumber"),
            "termStartDate": obj.get("termStartDate"),
            "termType": obj.get("termType")
        })
        return _obj


