# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.create_batch_query_response import CreateBatchQueryResponse
from typing import Optional, Set
from typing_extensions import Self

class CreateBatchQueryJobResponse(BaseModel):
    """
    CreateBatchQueryJobResponse
    """ # noqa: E501
    batches: Optional[List[CreateBatchQueryResponse]] = Field(default=None, description="A JSON array object that contains a list of batch objects. ")
    error_code: Optional[StrictStr] = Field(default=None, description="The error code used to identify the failure reason.", alias="errorCode")
    message: Optional[StrictStr] = Field(default=None, description="The error message used to describe the failure reason detail.")
    encrypted: Optional[StrictStr] = None
    format: Optional[StrictStr] = None
    id: Optional[StrictStr] = Field(default=None, description="The job ID created for the AQuA API request. The job ID can be used for querying for the query status.   The ID exists only if the JSON request can be parsed and validated successfully. Otherwise, the job ID is null. ")
    name: Optional[StrictStr] = Field(default=None, description="The name of the job. 32 character limit. ")
    notify_url: Optional[StrictStr] = Field(default=None, description="If URL is provided, the AQuA job will call this `notifyUrl` once the job has completed. The value of `notifyUrl` needs to have `${JOBID}` and `${STATUS}` placeholders. These placeholders will be replaced by the actual job ID and status when returned in the response. Status will be `Completed` after the AQuA job is done.  If you submit an AQuA query with `notifyUrl` specified, the value of `notifyUrl` will be ignored if your organization has already <a href=\"https://knowledgecenter.zuora.com/Zuora_Central_Platform/API/AB_Aggregate_Query_API/Callout_Notification_for_Completed_AQuA_Jobs\" target=\"_blank\">configured a callout notification through the Zuora user interface</a>.            ", alias="notifyUrl")
    offset: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="This field specifies the time offset for AQuA queries in stateful mode. It is an integer in the range 0 to 3,600 seconds.  For example, if you set this field to 600 seconds and you post a query in stateful mode at 2:00 AM, it will query against data created or updated between the completion time of the previous query and 1:50 AM.  The value of this field will override the value you configured in **Settings** > **Administration** > **AQuA API Stateful Mode Time Offset**.     ")
    partner: Optional[StrictStr] = Field(default=None, description="The partner field indicates the unique ID of a data integration partner. The dropdown list of this field displays partner IDs for the past thirty days.  It must be used together with \"project\" field to uniquely identify a data integration target.  For example, if a continuous AQuA session is to retrieve data incrementally for a Salesforce.com Org 00170000011K3Ub, you can use partner as \"Salesforce\", and \"project\" as \"00170000011K3Ub.\"   This field is required only if you are using AQuA in stateful mode. Otherwise, if you are using AQuA in stateless mode, partner field can be null.  **Note**: Zuora highly recommends you use the stateless mode instead of the stateful mode to extract bulk data. See <a href=\"https://knowledgecenter.zuora.com/Zuora_Central_Platform/API/AB_Aggregate_Query_API/Bulk_data__extraction_from_Zuora_using_AQuA\" target=\"_blank\">Bulk data extraction from Zuora using AQuA</a> for best practices.  **Note**: Submit a request at <a href=\"http://support.zuora.com\" target=\"_blank\">Zuora Global Support</a> to obtain a partner ID. ")
    project: Optional[StrictStr] = Field(default=None, description="The project field contains the unique ID of a data integration project for a particular partner. The dropdown list of this field displays project IDs for the past thirty days.  This field must be used together with partner field to uniquely identify a data integration target.   This field is required only if you are using AQuA in stateful mode. Otherwise, if you are using AQuA in stateless mode, partner field can be null. ")
    status: Optional[StrictStr] = None
    localized_status: Optional[StrictStr] = Field(default=None, alias="localizedStatus")
    use_last_completed_job_queries: Optional[StrictBool] = Field(default=None, description="If this flag is set to `true`, then all the previous queries are merged with existing queries.  If the flag is set to `false`, then the previous queries are ignored, and only the new query is executed. ", alias="useLastCompletedJobQueries")
    version: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The API version you want to use.   The supported versions are as follows:   - `1.1`. It supports both modes   - `1.0`. Default. It supports stateless modes only.  See <a href=\"https://knowledgecenter.zuora.com/Zuora_Central_Platform/API/AB_Aggregate_Query_API/BA_Stateless_and_Stateful_Modes\" target=\"_blank\">Stateless and stateful modes</a> for more information. ")
    __properties: ClassVar[List[str]] = ["batches", "errorCode", "message", "encrypted", "format", "id", "name", "notifyUrl", "offset", "partner", "project", "status", "localizedStatus", "useLastCompletedJobQueries", "version"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateBatchQueryJobResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in batches (list)
        _items = []
        if self.batches:
            for _item in self.batches:
                if _item:
                    _items.append(_item.to_dict())
            _dict['batches'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateBatchQueryJobResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "batches": [CreateBatchQueryResponse.from_dict(_item) for _item in obj["batches"]] if obj.get("batches") is not None else None,
            "errorCode": obj.get("errorCode"),
            "message": obj.get("message"),
            "encrypted": obj.get("encrypted"),
            "format": obj.get("format"),
            "id": obj.get("id"),
            "name": obj.get("name"),
            "notifyUrl": obj.get("notifyUrl"),
            "offset": obj.get("offset"),
            "partner": obj.get("partner"),
            "project": obj.get("project"),
            "status": obj.get("status"),
            "localizedStatus": obj.get("localizedStatus"),
            "useLastCompletedJobQueries": obj.get("useLastCompletedJobQueries"),
            "version": obj.get("version")
        })
        return _obj


