# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from zuora_sdk.models.bill_run_filter import BillRunFilter
from zuora_sdk.models.bill_run_schedule import BillRunSchedule
from typing import Optional, Set
from typing_extensions import Self

class CreateBillRunRequest(BaseModel):
    """
    CreateBillRunRequest
    """ # noqa: E501
    name: Optional[StrictStr] = Field(default=None, description="The name of the bill run. ")
    batches: Optional[List[StrictStr]] = Field(default=None, description="The batch of accounts for this bill run.   You can only specify either this field or the `billRunFilters` field.  **Values:** `AllBatches` or an array of `Batch*n*` where *n* is one of numbers 1 - 50, for example, `Batch7`. ")
    bill_cycle_day: Optional[StrictStr] = Field(default=None, description="The day of the bill cycle. This field is only valid if the `batches` field is specified.  **Values:**  - `AllBillCycleDays` or one of numbers 1 - 31 for an ad-hoc bill run - `AllBillCycleDays`, one of numbers 1 - 31, or `AsRunDay` for a scheduled bill run ", alias="billCycleDay")
    bill_run_filters: Optional[List[BillRunFilter]] = Field(default=None, description="The target account or subscriptions for this bill run. You can only specify either this field or the `batches` field. ", alias="billRunFilters")
    bill_run_type: Optional[StrictStr] = Field(default=None, alias="billRunType")
    charge_type_to_exclude: Optional[List[StrictStr]] = Field(default=None, description="The types of the charges to be excluded from the generation of billing documents. You can specify at most two charge types in the array. ", alias="chargeTypeToExclude")
    auto_email: Optional[StrictBool] = Field(default=False, description="Whether to automatically send emails after Auto-Post is complete.  **Note:** To use this field, you must first set the <a href=\"https://knowledgecenter.zuora.com/CB_Billing/Billing_Settings/Define_Billing_Rules\" target=\"_blank\">Support Bill Run Auto-Post?</a> billing rule to **Yes** through the Zuora UI. ", alias="autoEmail")
    auto_post: Optional[StrictBool] = Field(default=False, description="Whether to automatically post the bill run after the bill run is created.  **Note:** To use this field, you must first set the <a href=\"https://knowledgecenter.zuora.com/CB_Billing/Billing_Settings/Define_Billing_Rules\" target=\"_blank\">Support Bill Run Auto-Post?</a> billing rule to **Yes** through the Zuora UI. ", alias="autoPost")
    auto_renewal: Optional[StrictBool] = Field(default=False, description="Whether to automatically renew auto-renew subscriptions that are up for renewal. ", alias="autoRenewal")
    no_email_for_zero_amount_invoice: Optional[StrictBool] = Field(default=False, description="Whether to suppress emails for invoices with zero total amount generated in this bill run after the bill run is complete.   It is best practice to not send emails for invoices with zero amount. ", alias="noEmailForZeroAmountInvoice")
    schedule: Optional[BillRunSchedule] = None
    invoice_date: Optional[date] = Field(default=None, description="The invoice date for the bill run.   - When creating an ad-hoc bill run, if you do not specify any value for this field, the default value is the current date. - When creating a scheduled bill run, if you do not specify any value for this field, the invoice date is the value of the `repeatFrom` field. ", alias="invoiceDate")
    invoice_date_month_offset: Optional[StrictInt] = Field(default=None, description="The month offset of invoice date for this bill run based on run date, only valid for monthly scheduled bill runs. invoiceDateOffset and invoiceDateMonthOffset/invoiceDateDayOfMonth are mutually exclusive. invoiceDateMonthOffset and invoiceDateDayOfMonth coexist. ", alias="invoiceDateMonthOffset")
    invoice_date_day_of_month: Optional[Annotated[int, Field(le=31, strict=True, ge=1)]] = Field(default=None, description="The day of month of invoice date for this bill run, only valid for monthly scheduled bill runs. Specify any day of the month (1-31, where 31 = end-of-month). invoiceDateOffset and invoiceDateMonthOffset/invoiceDateDayOfMonth are mutually exclusive. invoiceDateMonthOffset and invoiceDateDayOfMonth coexist. ", alias="invoiceDateDayOfMonth")
    target_date: Optional[date] = Field(default=None, description="The target date for this bill run.   - You must specify this field when creating an ad-hoc bill run. - For scheduled bill runs, if you do not specify any value for this field, the target date is the value of the `repeatFrom` field. ", alias="targetDate")
    target_date_month_offset: Optional[StrictInt] = Field(default=None, description="The month offset of target date for this bill run based on run date, only valid for monthly scheduled bill runs. targetDateOffset and targetDateMonthOffset/targetDateDayOfMonth are mutually exclusive. targetDateMonthOffset and targetDateDayOfMonth coexist. ", alias="targetDateMonthOffset")
    target_date_day_of_month: Optional[Annotated[int, Field(le=31, strict=True, ge=1)]] = Field(default=None, description="The day of month of target date for this bill run, only valid for monthly scheduled bill runs. Specify any day of the month (1-31, where 31 = end-of-month). targetDateOffset and targetDateMonthOffset/targetDateDayOfMonth are mutually exclusive. targetDateMonthOffset and targetDateDayOfMonth coexist. ", alias="targetDateDayOfMonth")
    __properties: ClassVar[List[str]] = ["name", "batches", "billCycleDay", "billRunFilters", "billRunType", "chargeTypeToExclude", "autoEmail", "autoPost", "autoRenewal", "noEmailForZeroAmountInvoice", "schedule", "invoiceDate", "invoiceDateMonthOffset", "invoiceDateDayOfMonth", "targetDate", "targetDateMonthOffset", "targetDateDayOfMonth"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateBillRunRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in bill_run_filters (list)
        _items = []
        if self.bill_run_filters:
            for _item in self.bill_run_filters:
                if _item:
                    _items.append(_item.to_dict())
            _dict['billRunFilters'] = _items
        # override the default output from pydantic by calling `to_dict()` of schedule
        if self.schedule:
            _dict['schedule'] = self.schedule.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateBillRunRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "name": obj.get("name"),
            "batches": obj.get("batches"),
            "billCycleDay": obj.get("billCycleDay"),
            "billRunFilters": [BillRunFilter.from_dict(_item) for _item in obj["billRunFilters"]] if obj.get("billRunFilters") is not None else None,
            "billRunType": obj.get("billRunType"),
            "chargeTypeToExclude": obj.get("chargeTypeToExclude"),
            "autoEmail": obj.get("autoEmail") if obj.get("autoEmail") is not None else False,
            "autoPost": obj.get("autoPost") if obj.get("autoPost") is not None else False,
            "autoRenewal": obj.get("autoRenewal") if obj.get("autoRenewal") is not None else False,
            "noEmailForZeroAmountInvoice": obj.get("noEmailForZeroAmountInvoice") if obj.get("noEmailForZeroAmountInvoice") is not None else False,
            "schedule": BillRunSchedule.from_dict(obj["schedule"]) if obj.get("schedule") is not None else None,
            "invoiceDate": obj.get("invoiceDate"),
            "invoiceDateMonthOffset": obj.get("invoiceDateMonthOffset"),
            "invoiceDateDayOfMonth": obj.get("invoiceDateDayOfMonth"),
            "targetDate": obj.get("targetDate"),
            "targetDateMonthOffset": obj.get("targetDateMonthOffset"),
            "targetDateDayOfMonth": obj.get("targetDateDayOfMonth")
        })
        return _obj


