# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from zuora_sdk.models.billing_adjustment_exclusion import BillingAdjustmentExclusion
from typing import Optional, Set
from typing_extensions import Self

class CreateBillingAdjustmentRequest(BaseModel):
    """
    CreateBillingAdjustmentRequest
    """ # noqa: E501
    account_number: Optional[StrictStr] = Field(default=None, description="The account number for which the adjustment is created.    **Note**: The account number should be of the subscription owner.  **Note**: Only one of accountNumber or subscriptionNumber should be provided. ", alias="accountNumber")
    charge_numbers: Optional[List[StrictStr]] = Field(default=None, description="An optional container to specify charge numbers in the subscription for which the adjustment needs to be created. ", alias="chargeNumbers")
    deferred_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for the deferred revenue, such as Monthly Recurring Liability. ", alias="deferredRevenueAccountingCode")
    end_date: date = Field(description="The end date of the adjustment, in `yyyy-mm-dd` format. This is inclusive. ", alias="endDate")
    exclusion: Optional[List[BillingAdjustmentExclusion]] = Field(default=None, description="The charge numbers and the corresponding dates for exclusion of adjustment. ")
    reason: Optional[StrictStr] = Field(default=None, description="The reason for the adjustment. ")
    recognized_revenue_accounting_code: Optional[StrictStr] = Field(default=None, description="The accounting code for the recognized revenue, such as Monthly Recurring Charges or Overage Charges. ", alias="recognizedRevenueAccountingCode")
    revenue_recognition_rule_name: Optional[StrictStr] = Field(default=None, description="The name of the revenue recognition rule governing the revenue schedule. ", alias="revenueRecognitionRuleName")
    start_date: date = Field(description="The start date of the adjustment, in `yyyy-mm-dd` format. This is inclusive. ", alias="startDate")
    subscription_number: Optional[StrictStr] = Field(default=None, description="The subscription number for which the adjustment is created.  **Note**: Only one of accountNumber or subscriptionNumber should be provided. ", alias="subscriptionNumber")
    type: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["accountNumber", "chargeNumbers", "deferredRevenueAccountingCode", "endDate", "exclusion", "reason", "recognizedRevenueAccountingCode", "revenueRecognitionRuleName", "startDate", "subscriptionNumber", "type"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateBillingAdjustmentRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in exclusion (list)
        _items = []
        if self.exclusion:
            for _item in self.exclusion:
                if _item:
                    _items.append(_item.to_dict())
            _dict['exclusion'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateBillingAdjustmentRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "accountNumber": obj.get("accountNumber"),
            "chargeNumbers": obj.get("chargeNumbers"),
            "deferredRevenueAccountingCode": obj.get("deferredRevenueAccountingCode"),
            "endDate": obj.get("endDate"),
            "exclusion": [BillingAdjustmentExclusion.from_dict(_item) for _item in obj["exclusion"]] if obj.get("exclusion") is not None else None,
            "reason": obj.get("reason"),
            "recognizedRevenueAccountingCode": obj.get("recognizedRevenueAccountingCode"),
            "revenueRecognitionRuleName": obj.get("revenueRecognitionRuleName"),
            "startDate": obj.get("startDate"),
            "subscriptionNumber": obj.get("subscriptionNumber"),
            "type": obj.get("type")
        })
        return _obj


