# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class CreateContactRequest(BaseModel):
    """
    CreateContactRequest
    """ # noqa: E501
    account_id: Optional[StrictStr] = Field(default=None, description="The ID of the account associated with the contact.   **Note**: When creating a contact, you must specify `accountNumber`, `accountId`, or both in the request body. If both fields are specified, they must correspond to the same account. ", alias="accountId")
    account_number: Optional[StrictStr] = Field(default=None, description="The number of the customer account associated with the contact.   **Note**: When creating a contact, you must specify `accountNumber`, `accountId`, or both in the request body. If both fields are specified, they must correspond to the same account. ", alias="accountNumber")
    address1: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The first line of the contact's address, which is often a street address or business name. ")
    address2: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The second line of the contact's address. ")
    city: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="The city of the contact's address. ")
    contact_description: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="A description for the contact. ", alias="contactDescription")
    country: Optional[Annotated[str, Field(strict=True, max_length=64)]] = Field(default=None, description="The country of the contact's address. ")
    county: Optional[Annotated[str, Field(strict=True, max_length=32)]] = Field(default=None, description="The county. May optionally be used by Zuora Tax to calculate county tax. ")
    fax: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="The contact's fax number. ")
    first_name: Annotated[str, Field(strict=True, max_length=100)] = Field(description="The contact's first name. ", alias="firstName")
    home_phone: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="The contact's home phone number. ", alias="homePhone")
    last_name: Annotated[str, Field(strict=True, max_length=100)] = Field(description="The contact's last name. ", alias="lastName")
    mobile_phone: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The mobile phone number of the contact. ", alias="mobilePhone")
    nickname: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="A nickname for the contact. ")
    other_phone: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="An additional phone number for the contact. ", alias="otherPhone")
    other_phone_type: Optional[StrictStr] = Field(default=None, alias="otherPhoneType")
    personal_email: Optional[Annotated[str, Field(strict=True, max_length=80)]] = Field(default=None, description="The contact's personal email address. ", alias="personalEmail")
    state: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="The state or province of the contact's address. ")
    tax_region: Optional[Annotated[str, Field(strict=True, max_length=32)]] = Field(default=None, description="If using Zuora Tax, a region string as optionally defined in your tax rules. Not required. ", alias="taxRegion")
    work_email: Optional[Annotated[str, Field(strict=True, max_length=80)]] = Field(default=None, description="The contact's business email address. ", alias="workEmail")
    work_phone: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="The contact's business phone number. ", alias="workPhone")
    zip_code: Optional[Annotated[str, Field(strict=True, max_length=20)]] = Field(default=None, description="The zip code for the contact's address. ", alias="zipCode")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["accountId", "accountNumber", "address1", "address2", "city", "contactDescription", "country", "county", "fax", "firstName", "homePhone", "lastName", "mobilePhone", "nickname", "otherPhone", "otherPhoneType", "personalEmail", "state", "taxRegion", "workEmail", "workPhone", "zipCode"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateContactRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateContactRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "accountId": obj.get("accountId"),
            "accountNumber": obj.get("accountNumber"),
            "address1": obj.get("address1"),
            "address2": obj.get("address2"),
            "city": obj.get("city"),
            "contactDescription": obj.get("contactDescription"),
            "country": obj.get("country"),
            "county": obj.get("county"),
            "fax": obj.get("fax"),
            "firstName": obj.get("firstName"),
            "homePhone": obj.get("homePhone"),
            "lastName": obj.get("lastName"),
            "mobilePhone": obj.get("mobilePhone"),
            "nickname": obj.get("nickname"),
            "otherPhone": obj.get("otherPhone"),
            "otherPhoneType": obj.get("otherPhoneType"),
            "personalEmail": obj.get("personalEmail"),
            "state": obj.get("state"),
            "taxRegion": obj.get("taxRegion"),
            "workEmail": obj.get("workEmail"),
            "workPhone": obj.get("workPhone"),
            "zipCode": obj.get("zipCode")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


