# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from zuora_sdk.models.create_journal_entry_request_item import CreateJournalEntryRequestItem
from zuora_sdk.models.create_journal_entry_request_segment import CreateJournalEntryRequestSegment
from zuora_sdk.models.transferred_to_accounting_status import TransferredToAccountingStatus
from typing import Optional, Set
from typing_extensions import Self

class CreateJournalEntryRequest(BaseModel):
    """
    CreateJournalEntryRequest
    """ # noqa: E501
    accounting_period_name: StrictStr = Field(description="Name of the accounting period. The open-ended accounting period is named `Open-Ended`. ", alias="accountingPeriodName")
    currency: StrictStr = Field(description="The type of currency used. Currency must be active. ")
    journal_entry_date: date = Field(description="Date of the journal entry. ", alias="journalEntryDate")
    journal_entry_items: List[CreateJournalEntryRequestItem] = Field(description="Key name that represents the list of journal entry items. ", alias="journalEntryItems")
    notes: Optional[StrictStr] = Field(default=None, description="The number associated with the revenue event.  Character limit: 2,000 ")
    organization_label: Optional[StrictStr] = Field(default=None, description="Organization Label ", alias="organizationLabel")
    segments: Optional[List[CreateJournalEntryRequestSegment]] = Field(default=None, description="List of segments that apply to the summary journal entry. ")
    transferred_to_accounting: Optional[TransferredToAccountingStatus] = Field(default=None, alias="transferredToAccounting")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["accountingPeriodName", "currency", "journalEntryDate", "journalEntryItems", "notes", "organizationLabel", "segments", "transferredToAccounting"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateJournalEntryRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in journal_entry_items (list)
        _items = []
        if self.journal_entry_items:
            for _item in self.journal_entry_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['journalEntryItems'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in segments (list)
        _items = []
        if self.segments:
            for _item in self.segments:
                if _item:
                    _items.append(_item.to_dict())
            _dict['segments'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateJournalEntryRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "accountingPeriodName": obj.get("accountingPeriodName"),
            "currency": obj.get("currency"),
            "journalEntryDate": obj.get("journalEntryDate"),
            "journalEntryItems": [CreateJournalEntryRequestItem.from_dict(_item) for _item in obj["journalEntryItems"]] if obj.get("journalEntryItems") is not None else None,
            "notes": obj.get("notes"),
            "organizationLabel": obj.get("organizationLabel"),
            "segments": [CreateJournalEntryRequestSegment.from_dict(_item) for _item in obj["segments"]] if obj.get("segments") is not None else None,
            "transferredToAccounting": obj.get("transferredToAccounting")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


