# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from zuora_sdk.models.charge_override import ChargeOverride
from zuora_sdk.models.create_order_rate_plan_feature_override import CreateOrderRatePlanFeatureOverride
from typing import Optional, Set
from typing_extensions import Self

class CreateOrderProductRatePlanOverride(BaseModel):
    """
    Information about an order action of type `addProduct`.  
    """ # noqa: E501
    charge_overrides: Optional[List[ChargeOverride]] = Field(default=None, description="List of charges associated with the rate plan. ", alias="chargeOverrides")
    clearing_existing_features: Optional[StrictBool] = Field(default=None, description="Specifies whether all features in the rate plan will be cleared. ", alias="clearingExistingFeatures")
    custom_fields: Optional[Dict[str, Dict[str, Any]]] = Field(default=None, description="Container for custom fields of the Rate Plan object. The custom fields of the Rate Plan object are used when rate plans are subscribed. ", alias="customFields")
    external_catalog_plan_id: Optional[StrictStr] = Field(default=None, description="An external ID of the product rate plan to be added. You can use this field to specify a product rate plan that is imported from an external system. The value of the `externalCatalogPlanId` field must match one of the values that are predefined in the `externallyManagedPlanIds` field on a product rate plan.  **Note:** If both `externalCatalogPlanId` and `productRatePlanId` are provided. They must point to the same product rate plan. Otherwise, the request would fail. ", alias="externalCatalogPlanId")
    externally_managed_plan_id: Optional[StrictStr] = Field(default=None, description="Indicates the unique identifier for the rate plan purchased on a third-party store. This field is used to represent a subscription rate plan created through third-party stores. ", alias="externallyManagedPlanId")
    product_rate_plan_id: Optional[StrictStr] = Field(default=None, description="Internal identifier of the product rate plan that the rate plan is based on. ", alias="productRatePlanId")
    product_rate_plan_number: Optional[StrictStr] = Field(default=None, description="Number of a product rate plan for this subscription. ", alias="productRatePlanNumber")
    subscription_product_features: Optional[List[CreateOrderRatePlanFeatureOverride]] = Field(default=None, description="List of features associated with the rate plan. The system compares the `subscriptionProductFeatures` and `featureId` fields in the request with the counterpart fields in a rate plan. The comparison results are as follows: * If there is no `subscriptionProductFeatures` field or the field is empty, features in the rate plan remain unchanged. But if the `clearingExistingFeatures` field is additionally set to true, all features in the rate plan are cleared. * If the `subscriptionProductFeatures` field contains the `featureId` nested fields, as well as the optional `description` and `customFields` nested fields, the features indicated by the featureId nested fields in the request overwrite all features in the rate plan. ", alias="subscriptionProductFeatures")
    unique_token: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="Unique identifier for the rate plan. This identifier enables you to refer to the rate plan before the rate plan has an internal identifier in Zuora.  For instance, suppose that you want to use a single order to add a product to a subscription and later update the same product. When you add the product, you can set a unique identifier for the rate plan. Then when you update the product, you can use the same unique identifier to specify which rate plan to modify. ", alias="uniqueToken")
    __properties: ClassVar[List[str]] = ["chargeOverrides", "clearingExistingFeatures", "customFields", "externalCatalogPlanId", "externallyManagedPlanId", "productRatePlanId", "productRatePlanNumber", "subscriptionProductFeatures", "uniqueToken"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateOrderProductRatePlanOverride from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in charge_overrides (list)
        _items = []
        if self.charge_overrides:
            for _item in self.charge_overrides:
                if _item:
                    _items.append(_item.to_dict())
            _dict['chargeOverrides'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in subscription_product_features (list)
        _items = []
        if self.subscription_product_features:
            for _item in self.subscription_product_features:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subscriptionProductFeatures'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateOrderProductRatePlanOverride from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "chargeOverrides": [ChargeOverride.from_dict(_item) for _item in obj["chargeOverrides"]] if obj.get("chargeOverrides") is not None else None,
            "clearingExistingFeatures": obj.get("clearingExistingFeatures"),
            "customFields": obj.get("customFields"),
            "externalCatalogPlanId": obj.get("externalCatalogPlanId"),
            "externallyManagedPlanId": obj.get("externallyManagedPlanId"),
            "productRatePlanId": obj.get("productRatePlanId"),
            "productRatePlanNumber": obj.get("productRatePlanNumber"),
            "subscriptionProductFeatures": [CreateOrderRatePlanFeatureOverride.from_dict(_item) for _item in obj["subscriptionProductFeatures"]] if obj.get("subscriptionProductFeatures") is not None else None,
            "uniqueToken": obj.get("uniqueToken")
        })
        return _obj


