# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.override_discount_apply_detail import OverrideDiscountApplyDetail
from typing import Optional, Set
from typing_extensions import Self

class DiscountPricingOverride(BaseModel):
    """
    Pricing information about a discount charge. 
    """ # noqa: E501
    apply_discount_to: Optional[StrictStr] = Field(default=None, alias="applyDiscountTo")
    original_discount_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The manufacturer's suggested retail discount price for standalone charge.  Only applicable if the standalone discount charge is a fixed-amount discount.  **Note:** This field is available when the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Manage_subscription_transactions/Orders/Standalone_Orders/AA_Overview_of_Standalone_Orders\" target=\"_blank\">Standalone Orders</a> feature is enabled. ", alias="originalDiscountAmount")
    discount_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Only applicable if the discount charge is a fixed-amount discount. ", alias="discountAmount")
    discount_level: Optional[StrictStr] = Field(default=None, alias="discountLevel")
    original_discount_percentage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The manufacturer's suggested retail discount percentage for standalone charge.  Only applicable if the standalone discount charge is a percentage discount.  **Note:** This field is available when the <a href=\"https://knowledgecenter.zuora.com/Zuora_Billing/Manage_subscription_transactions/Orders/Standalone_Orders/AA_Overview_of_Standalone_Orders\" target=\"_blank\">Standalone Orders</a> feature is enabled. ", alias="originalDiscountPercentage")
    discount_percentage: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Only applicable if the discount charge is a percentage discount. ", alias="discountPercentage")
    price_change_option: Optional[StrictStr] = Field(default=None, alias="priceChangeOption")
    discount_class: Optional[StrictStr] = Field(default=None, description="The discount class defines the sequence in which discount product rate plan charges are applied.  **Note**: You must enable the [Enhanced Discounts](https://knowledgecenter.zuora.com/Zuora_Billing/Build_products_and_prices/Basic_concepts_and_terms/B_Charge_Models/D_Manage_Enhanced_Discount) feature to access this field. ", alias="discountClass")
    apply_to_billing_period_partially: Optional[StrictBool] = Field(default=None, description="Allow the discount duration to be aligned with the billing period partially.  **Note**: You must enable the [Enhanced Discounts](https://knowledgecenter.zuora.com/Zuora_Billing/Build_products_and_prices/Basic_concepts_and_terms/B_Charge_Models/D_Manage_Enhanced_Discount) feature to access this field. ", alias="applyToBillingPeriodPartially")
    discount_apply_details: Optional[List[OverrideDiscountApplyDetail]] = Field(default=None, description="Charge list of discount be applied to.  **Note**: You must enable the [Enhanced Discounts](https://knowledgecenter.zuora.com/Zuora_Billing/Build_products_and_prices/Basic_concepts_and_terms/B_Charge_Models/D_Manage_Enhanced_Discount) feature to access this field. ", alias="discountApplyDetails")
    __properties: ClassVar[List[str]] = ["applyDiscountTo", "originalDiscountAmount", "discountAmount", "discountLevel", "originalDiscountPercentage", "discountPercentage", "priceChangeOption", "discountClass", "applyToBillingPeriodPartially", "discountApplyDetails"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DiscountPricingOverride from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in discount_apply_details (list)
        _items = []
        if self.discount_apply_details:
            for _item in self.discount_apply_details:
                if _item:
                    _items.append(_item.to_dict())
            _dict['discountApplyDetails'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DiscountPricingOverride from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "applyDiscountTo": obj.get("applyDiscountTo"),
            "originalDiscountAmount": obj.get("originalDiscountAmount"),
            "discountAmount": obj.get("discountAmount"),
            "discountLevel": obj.get("discountLevel"),
            "originalDiscountPercentage": obj.get("originalDiscountPercentage"),
            "discountPercentage": obj.get("discountPercentage"),
            "priceChangeOption": obj.get("priceChangeOption"),
            "discountClass": obj.get("discountClass"),
            "applyToBillingPeriodPartially": obj.get("applyToBillingPeriodPartially"),
            "discountApplyDetails": [OverrideDiscountApplyDetail.from_dict(_item) for _item in obj["discountApplyDetails"]] if obj.get("discountApplyDetails") is not None else None
        })
        return _obj


