# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.fulfillment_state import FulfillmentState
from zuora_sdk.models.fulfillment_type import FulfillmentType
from zuora_sdk.models.get_fulfillment_item import GetFulfillmentItem
from typing import Optional, Set
from typing_extensions import Self

class GetFulfillment(BaseModel):
    """
    GetFulfillment
    """ # noqa: E501
    bill_target_date: Optional[date] = Field(default=None, description="The target date for the Fulfillment to be picked up by bill run for billing. ", alias="billTargetDate")
    carrier: Optional[StrictStr] = Field(default=None, description="The carrier of the Fulfillment. The available values can be configured in **Billing Settings** > **Fulfillment Settings** through Zuora UI. ")
    custom_fields: Optional[Dict[str, Dict[str, Any]]] = Field(default=None, description="Container for custom fields of a Fulfillment object. ", alias="customFields")
    description: Optional[StrictStr] = Field(default=None, description="The description of the Fulfillment. ")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(default=None, description="The flag to exclude Fulfillment related invoice items, invoice item adjustments, credit memo items, and debit memo items from revenue accounting.  **Note**: This field is only available if you have the [Zuora Billing - Revenue Integration](https://knowledgecenter.zuora.com/Zuora_Revenue/Zuora_Billing_-_Revenue_Integration) feature enabled.  ", alias="excludeItemBillingFromRevenueAccounting")
    exclude_item_booking_from_revenue_accounting: Optional[StrictBool] = Field(default=None, description="The flag to exclude Fulfillment from revenue accounting.  **Note**: This field is only available if you have the [Zuora Billing - Revenue Integration](https://knowledgecenter.zuora.com/Zuora_Revenue/Zuora_Billing_-_Revenue_Integration) feature enabled.  ", alias="excludeItemBookingFromRevenueAccounting")
    external_id: Optional[StrictStr] = Field(default=None, description="The external id of the Fulfillment. ", alias="externalId")
    fulfillment_date: Optional[date] = Field(default=None, description="The date of the Fulfillment. ", alias="fulfillmentDate")
    fulfillment_location: Optional[StrictStr] = Field(default=None, description="The fulfillment location of the Fulfillment. The available values can be configured in **Billing Settings** > **Fulfillment Settings** through Zuora UI. ", alias="fulfillmentLocation")
    fulfillment_system: Optional[StrictStr] = Field(default=None, description="The fulfillment system of the Fulfillment. The available values can be configured in **Billing Settings** > **Fulfillment Settings** through Zuora UI. ", alias="fulfillmentSystem")
    fulfillment_type: Optional[StrictStr] = Field(default=None, alias="fulfillmentType")
    order_line_item_id: Optional[StrictStr] = Field(default=None, description="The reference id of the related Order Line Item. ", alias="orderLineItemId")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The quantity of the Fulfillment. ")
    state: Optional[StrictStr] = None
    tracking_number: Optional[StrictStr] = Field(default=None, description="The tracking number of the Fulfillment. ", alias="trackingNumber")
    fulfillment_items: Optional[List[GetFulfillmentItem]] = Field(default=None, description="The fulfillmentItems of the Fulfillment", alias="fulfillmentItems")
    fulfillment_number: Optional[StrictStr] = Field(default=None, description="The sytem generated number for the Fulfillment. ", alias="fulfillmentNumber")
    id: Optional[StrictStr] = Field(default=None, description="The system generated Id. ")
    __properties: ClassVar[List[str]] = ["billTargetDate", "carrier", "customFields", "description", "excludeItemBillingFromRevenueAccounting", "excludeItemBookingFromRevenueAccounting", "externalId", "fulfillmentDate", "fulfillmentLocation", "fulfillmentSystem", "fulfillmentType", "orderLineItemId", "quantity", "state", "trackingNumber", "fulfillmentItems", "fulfillmentNumber", "id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetFulfillment from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in fulfillment_items (list)
        _items = []
        if self.fulfillment_items:
            for _item in self.fulfillment_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['fulfillmentItems'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetFulfillment from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "billTargetDate": obj.get("billTargetDate"),
            "carrier": obj.get("carrier"),
            "customFields": obj.get("customFields"),
            "description": obj.get("description"),
            "excludeItemBillingFromRevenueAccounting": obj.get("excludeItemBillingFromRevenueAccounting"),
            "excludeItemBookingFromRevenueAccounting": obj.get("excludeItemBookingFromRevenueAccounting"),
            "externalId": obj.get("externalId"),
            "fulfillmentDate": obj.get("fulfillmentDate"),
            "fulfillmentLocation": obj.get("fulfillmentLocation"),
            "fulfillmentSystem": obj.get("fulfillmentSystem"),
            "fulfillmentType": obj.get("fulfillmentType"),
            "orderLineItemId": obj.get("orderLineItemId"),
            "quantity": obj.get("quantity"),
            "state": obj.get("state"),
            "trackingNumber": obj.get("trackingNumber"),
            "fulfillmentItems": [GetFulfillmentItem.from_dict(_item) for _item in obj["fulfillmentItems"]] if obj.get("fulfillmentItems") is not None else None,
            "fulfillmentNumber": obj.get("fulfillmentNumber"),
            "id": obj.get("id")
        })
        return _obj


