# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.get_subscription_rate_plan_charges_with_all_segments import GetSubscriptionRatePlanChargesWithAllSegments
from zuora_sdk.models.subscription_product_feature import SubscriptionProductFeature
from typing import Optional, Set
from typing_extensions import Self

class GetSubscriptionRatePlan(BaseModel):
    """
    GetSubscriptionRatePlan
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="Rate plan ID. ")
    subscription_rate_plan_number: Optional[StrictStr] = Field(default=None, alias="subscriptionRatePlanNumber")
    is_from_external_catalog: Optional[StrictBool] = Field(default=None, alias="isFromExternalCatalog")
    last_change_type: Optional[StrictStr] = Field(default=None, description="The last amendment on the rate plan.    **Note:** If a subscription is created through an order, this field is only available if multiple orders are created on the subscription.   Possible Values:   * `Add`   * `Update`   * `Remove` ", alias="lastChangeType")
    product_id: Optional[StrictStr] = Field(default=None, alias="productId")
    product_name: Optional[StrictStr] = Field(default=None, alias="productName")
    product_sku: Optional[StrictStr] = Field(default=None, description="The unique SKU for the product.", alias="productSku")
    product_rate_plan_id: Optional[StrictStr] = Field(default=None, alias="productRatePlanId")
    product_rate_plan_number: Optional[StrictStr] = Field(default=None, alias="productRatePlanNumber")
    rate_plan_name: Optional[StrictStr] = Field(default=None, description="Name of the rate plan.", alias="ratePlanName")
    subscription_product_features: Optional[List[SubscriptionProductFeature]] = Field(default=None, description="Container for one or more features.   Only available when the following settings are enabled:  * The Entitlements feature in your tenant.  * The Enable Feature Specification in Product and Subscriptions setting in Zuora Billing Settings", alias="subscriptionProductFeatures")
    externally_managed_plan_id: Optional[StrictStr] = Field(default=None, description="Indicates the unique identifier for the rate plan purchased on a third-party store. This field is used to represent a subscription rate plan created through third-party stores. ", alias="externallyManagedPlanId")
    contracted_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Monthly recurring revenue of the subscription rate plan exclusive of all the discounts applicable. ", alias="contractedMrr")
    contracted_net_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Monthly recurring revenue of the subscription rate plan inclusive of all the discounts applicable, including the fixed-amount discounts and percentage discounts. ", alias="contractedNetMrr")
    as_of_day_gross_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Monthly recurring revenue of the subscription rate plan exclusive of any discounts applicable as of specified day. ", alias="asOfDayGrossMrr")
    as_of_day_net_mrr: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="Monthly recurring revenue of the subscription rate plan inclusive of all the discounts applicable, including the fixed-amount discounts and percentage discounts as of specified day. ", alias="asOfDayNetMrr")
    rate_plan_charges: Optional[List[GetSubscriptionRatePlanChargesWithAllSegments]] = Field(default=None, description="Container for one or more charges.", alias="ratePlanCharges")
    __properties: ClassVar[List[str]] = ["id", "subscriptionRatePlanNumber", "isFromExternalCatalog", "lastChangeType", "productId", "productName", "productSku", "productRatePlanId", "productRatePlanNumber", "ratePlanName", "subscriptionProductFeatures", "externallyManagedPlanId", "contractedMrr", "contractedNetMrr", "asOfDayGrossMrr", "asOfDayNetMrr", "ratePlanCharges"]

    @field_validator('last_change_type')
    def last_change_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Add', 'Update', 'Remove']):
            raise ValueError("must be one of enum values ('Add', 'Update', 'Remove')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetSubscriptionRatePlan from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in subscription_product_features (list)
        _items = []
        if self.subscription_product_features:
            for _item in self.subscription_product_features:
                if _item:
                    _items.append(_item.to_dict())
            _dict['subscriptionProductFeatures'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in rate_plan_charges (list)
        _items = []
        if self.rate_plan_charges:
            for _item in self.rate_plan_charges:
                if _item:
                    _items.append(_item.to_dict())
            _dict['ratePlanCharges'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetSubscriptionRatePlan from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "subscriptionRatePlanNumber": obj.get("subscriptionRatePlanNumber"),
            "isFromExternalCatalog": obj.get("isFromExternalCatalog"),
            "lastChangeType": obj.get("lastChangeType"),
            "productId": obj.get("productId"),
            "productName": obj.get("productName"),
            "productSku": obj.get("productSku"),
            "productRatePlanId": obj.get("productRatePlanId"),
            "productRatePlanNumber": obj.get("productRatePlanNumber"),
            "ratePlanName": obj.get("ratePlanName"),
            "subscriptionProductFeatures": [SubscriptionProductFeature.from_dict(_item) for _item in obj["subscriptionProductFeatures"]] if obj.get("subscriptionProductFeatures") is not None else None,
            "externallyManagedPlanId": obj.get("externallyManagedPlanId"),
            "contractedMrr": obj.get("contractedMrr"),
            "contractedNetMrr": obj.get("contractedNetMrr"),
            "asOfDayGrossMrr": obj.get("asOfDayGrossMrr"),
            "asOfDayNetMrr": obj.get("asOfDayNetMrr"),
            "ratePlanCharges": [GetSubscriptionRatePlanChargesWithAllSegments.from_dict(_item) for _item in obj["ratePlanCharges"]] if obj.get("ratePlanCharges") is not None else None
        })
        return _obj


