# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from decimal import Decimal
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing import Optional, Set
from typing_extensions import Self

class InvoiceTaxationItem(BaseModel):
    """
    InvoiceTaxationItem
    """ # noqa: E501
    available_to_credit_amount: Optional[Decimal] = Field(default=None, description="The amount of the invoice taxation item that is available to credit.", alias="availableToCreditAmount")
    balance: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The balance of the taxation item.")
    credit_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of credit memos applied to the taxation item.  ", alias="creditAmount")
    exempt_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The calculated tax amount excluded due to the exemption.", alias="exemptAmount")
    id: Optional[StrictStr] = Field(default=None, description="The ID of the taxation item.")
    jurisdiction: Optional[StrictStr] = Field(default=None, description="The jurisdiction that applies the tax or VAT. This value is typically a state, province, county, or city.")
    location_code: Optional[StrictStr] = Field(default=None, description="The identifier for the location based on the value of the `taxCode` field.", alias="locationCode")
    name: Optional[StrictStr] = Field(default=None, description="The name of the taxation item.")
    payment_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of payments applied to the taxation item.", alias="paymentAmount")
    tax_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of taxation.", alias="taxAmount")
    tax_code: Optional[StrictStr] = Field(default=None, description="The tax code identifies which tax rules and tax rates to apply to a specific invoice.", alias="taxCode")
    tax_code_description: Optional[StrictStr] = Field(default=None, description="The description of the tax code.", alias="taxCodeDescription")
    tax_date: Optional[date] = Field(default=None, description="The date that the tax is applied to the invoice, in `yyyy-mm-dd`format.", alias="taxDate")
    tax_rate: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The tax rate applied to the invoice.", alias="taxRate")
    tax_rate_description: Optional[StrictStr] = Field(default=None, description="The description of the tax rate.", alias="taxRateDescription")
    tax_rate_type: Optional[StrictStr] = Field(default=None, alias="taxRateType")
    __properties: ClassVar[List[str]] = ["availableToCreditAmount", "balance", "creditAmount", "exemptAmount", "id", "jurisdiction", "locationCode", "name", "paymentAmount", "taxAmount", "taxCode", "taxCodeDescription", "taxDate", "taxRate", "taxRateDescription", "taxRateType"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of InvoiceTaxationItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of InvoiceTaxationItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "availableToCreditAmount": obj.get("availableToCreditAmount"),
            "balance": obj.get("balance"),
            "creditAmount": obj.get("creditAmount"),
            "exemptAmount": obj.get("exemptAmount"),
            "id": obj.get("id"),
            "jurisdiction": obj.get("jurisdiction"),
            "locationCode": obj.get("locationCode"),
            "name": obj.get("name"),
            "paymentAmount": obj.get("paymentAmount"),
            "taxAmount": obj.get("taxAmount"),
            "taxCode": obj.get("taxCode"),
            "taxCodeDescription": obj.get("taxCodeDescription"),
            "taxDate": obj.get("taxDate"),
            "taxRate": obj.get("taxRate"),
            "taxRateDescription": obj.get("taxRateDescription"),
            "taxRateType": obj.get("taxRateType")
        })
        return _obj


