# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from zuora_sdk.models.time_sliced_elp_net_metrics import TimeSlicedElpNetMetrics
from zuora_sdk.models.time_sliced_metrics import TimeSlicedMetrics
from zuora_sdk.models.time_sliced_net_metrics import TimeSlicedNetMetrics
from zuora_sdk.models.time_sliced_tcb_net_metrics import TimeSlicedTcbNetMetrics
from typing import Optional, Set
from typing_extensions import Self

class OrderMetric(BaseModel):
    """
    The set of order metrics for an order action.
    """ # noqa: E501
    charge_number: Optional[StrictStr] = Field(default=None, alias="chargeNumber")
    elp: Optional[List[TimeSlicedElpNetMetrics]] = Field(default=None, description="The extended list price which is calculated by the original product catalog list price multiplied by the delta quantity.  The `elp` nested field is only available to existing Orders customers who already have access to the field.  **Note:** The following Order Metrics have been deprecated. Any new customers who onboard on [Orders](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/AA_Overview_of_Orders) or [Orders Harmonization](https://knowledgecenter.zuora.com/Billing/Subscriptions/Orders/Orders_Harmonization/Orders_Harmonization) will not get these metrics. * The Order ELP and Order Item objects  * The \"Generated Reason\" and \"Order Item ID\" fields in the Order MRR, Order TCB, Order TCV, and Order Quantity objects  Existing Orders customers who have these metrics will continue to be supported. ")
    mrr: Optional[List[TimeSlicedNetMetrics]] = None
    origin_rate_plan_id: Optional[StrictStr] = Field(default=None, alias="originRatePlanId")
    product_rate_plan_charge_id: Optional[StrictStr] = Field(default=None, alias="productRatePlanChargeId")
    product_rate_plan_id: Optional[StrictStr] = Field(default=None, alias="productRatePlanId")
    subscription_rate_plan_number: Optional[StrictStr] = Field(default=None, alias="subscriptionRatePlanNumber")
    quantity: Optional[List[TimeSlicedMetrics]] = None
    tcb: Optional[List[TimeSlicedTcbNetMetrics]] = Field(default=None, description="Total contracted billing which is the forecast value for the total invoice amount.")
    tcv: Optional[List[TimeSlicedNetMetrics]] = Field(default=None, description="Total contracted value.")
    __properties: ClassVar[List[str]] = ["chargeNumber", "elp", "mrr", "originRatePlanId", "productRatePlanChargeId", "productRatePlanId", "subscriptionRatePlanNumber", "quantity", "tcb", "tcv"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of OrderMetric from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in elp (list)
        _items = []
        if self.elp:
            for _item in self.elp:
                if _item:
                    _items.append(_item.to_dict())
            _dict['elp'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in mrr (list)
        _items = []
        if self.mrr:
            for _item in self.mrr:
                if _item:
                    _items.append(_item.to_dict())
            _dict['mrr'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in quantity (list)
        _items = []
        if self.quantity:
            for _item in self.quantity:
                if _item:
                    _items.append(_item.to_dict())
            _dict['quantity'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tcb (list)
        _items = []
        if self.tcb:
            for _item in self.tcb:
                if _item:
                    _items.append(_item.to_dict())
            _dict['tcb'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in tcv (list)
        _items = []
        if self.tcv:
            for _item in self.tcv:
                if _item:
                    _items.append(_item.to_dict())
            _dict['tcv'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of OrderMetric from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "chargeNumber": obj.get("chargeNumber"),
            "elp": [TimeSlicedElpNetMetrics.from_dict(_item) for _item in obj["elp"]] if obj.get("elp") is not None else None,
            "mrr": [TimeSlicedNetMetrics.from_dict(_item) for _item in obj["mrr"]] if obj.get("mrr") is not None else None,
            "originRatePlanId": obj.get("originRatePlanId"),
            "productRatePlanChargeId": obj.get("productRatePlanChargeId"),
            "productRatePlanId": obj.get("productRatePlanId"),
            "subscriptionRatePlanNumber": obj.get("subscriptionRatePlanNumber"),
            "quantity": [TimeSlicedMetrics.from_dict(_item) for _item in obj["quantity"]] if obj.get("quantity") is not None else None,
            "tcb": [TimeSlicedTcbNetMetrics.from_dict(_item) for _item in obj["tcb"]] if obj.get("tcb") is not None else None,
            "tcv": [TimeSlicedNetMetrics.from_dict(_item) for _item in obj["tcv"]] if obj.get("tcv") is not None else None
        })
        return _obj


