# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class PaymentMethodResponseMandateInfo(BaseModel):
    """
    The mandate information for the Credit Card, Apple Pay, Google Pay, Credit Card Reference Transaction, ACH, or Bank Transfer payment method.  The following mandate fields are common to all supported payment methods: * `mandateId` * `mandateReason` * `mandateStatus`  The following mandate fields are specific to the ACH and Bank Transfer payment methods: * `mandateReceivedStatus` * `existingMandateStatus` * `mandateCreationDate` * `mandateUpdateDate`  The following mandate fields are specific to the Credit Card, Apple Pay, and Google Pay payment methods: * `mitTransactionId` * `mitProfileAgreedOn` * `mitConsentAgreementRef` * `mitConsentAgreementSrc` * `mitProfileType` * `mitProfileAction` 
    """ # noqa: E501
    existing_mandate_status: Optional[StrictStr] = Field(default=None, alias="existingMandateStatus")
    mandate_creation_date: Optional[date] = Field(default=None, description="The date on which the mandate was created. ", alias="mandateCreationDate")
    mandate_id: Optional[StrictStr] = Field(default=None, description="The mandate ID. ", alias="mandateId")
    mandate_reason: Optional[StrictStr] = Field(default=None, description="The reason of the mandate from the gateway side. ", alias="mandateReason")
    mandate_received_status: Optional[StrictStr] = Field(default=None, alias="mandateReceivedStatus")
    mandate_status: Optional[StrictStr] = Field(default=None, description="The status of the mandate from the gateway side. ", alias="mandateStatus")
    mandate_update_date: Optional[date] = Field(default=None, description="The date on which the mandate was updated. ", alias="mandateUpdateDate")
    mit_consent_agreement_ref: Optional[StrictStr] = Field(default=None, description="Reference for the consent agreement that you have established with the customer.   ", alias="mitConsentAgreementRef")
    mit_consent_agreement_src: Optional[StrictStr] = Field(default=None, alias="mitConsentAgreementSrc")
    mit_profile_action: Optional[StrictStr] = Field(default=None, alias="mitProfileAction")
    mit_profile_agreed_on: Optional[date] = Field(default=None, description="The date on which the stored credential profile is agreed. The date format is `yyyy-mm-dd`. ", alias="mitProfileAgreedOn")
    mit_profile_type: Optional[StrictStr] = Field(default=None, description="Indicates the type of the stored credential profile. If you do not specify the `mitProfileAction` field, Zuora will automatically create a stored credential profile for the payment method, with the default value `Recurring` set to this field. ", alias="mitProfileType")
    mit_transaction_id: Optional[Annotated[str, Field(strict=True, max_length=128)]] = Field(default=None, description="Specifies the ID of the transaction. Only applicable if you set the `mitProfileAction` field to `Persist`. ", alias="mitTransactionId")
    __properties: ClassVar[List[str]] = ["existingMandateStatus", "mandateCreationDate", "mandateId", "mandateReason", "mandateReceivedStatus", "mandateStatus", "mandateUpdateDate", "mitConsentAgreementRef", "mitConsentAgreementSrc", "mitProfileAction", "mitProfileAgreedOn", "mitProfileType", "mitTransactionId"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentMethodResponseMandateInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentMethodResponseMandateInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "existingMandateStatus": obj.get("existingMandateStatus"),
            "mandateCreationDate": obj.get("mandateCreationDate"),
            "mandateId": obj.get("mandateId"),
            "mandateReason": obj.get("mandateReason"),
            "mandateReceivedStatus": obj.get("mandateReceivedStatus"),
            "mandateStatus": obj.get("mandateStatus"),
            "mandateUpdateDate": obj.get("mandateUpdateDate"),
            "mitConsentAgreementRef": obj.get("mitConsentAgreementRef"),
            "mitConsentAgreementSrc": obj.get("mitConsentAgreementSrc"),
            "mitProfileAction": obj.get("mitProfileAction"),
            "mitProfileAgreedOn": obj.get("mitProfileAgreedOn"),
            "mitProfileType": obj.get("mitProfileType"),
            "mitTransactionId": obj.get("mitTransactionId")
        })
        return _obj


