# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class PostBusinessRegionsRequest(BaseModel):
    """
    PostBusinessRegionsRequest
    """ # noqa: E501
    country: StrictStr = Field(description="The short name of a country or region where you must comply with e-invoicing requirements. For example, `IN` for India. For the full list of country names and codes, see <a href=\"https://knowledgecenter.zuora.com/Quick_References/Country%2C_State%2C_and_Province_Codes/A_Country_Names_and_Their_ISO_Codes\" target=\"_blank\">ISO Standard Country Codes</a>. ")
    business_name: Annotated[str, Field(strict=True, max_length=255)] = Field(description="The full official name that the Seller is registered with the relevant legal authority. ", alias="businessName")
    business_number: Optional[StrictStr] = Field(default=None, description="The specify the unique identifier number of the legal entity or person that you do business with.  For example, you must use a GSTIN for India and Tax Identification Number (TIN) for Saudi Arabia. ", alias="businessNumber")
    business_number_schema_id: Optional[StrictStr] = Field(default=None, description="The identification scheme identifier that an official registrar issues to identify the Seller as a legal entity or person. ", alias="businessNumberSchemaId")
    trade_name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The name that the Seller is known as, other than the legal business name. ", alias="tradeName")
    tax_register_number: Optional[StrictStr] = Field(default=None, description="The Seller's VAT identifier (also known as Seller VAT identification number) or the local identification (defined by the Seller’s address) of the Seller for tax purposes, or a reference that enables the Seller to state the registered tax status. ", alias="taxRegisterNumber")
    endpoint_id: Optional[StrictStr] = Field(default=None, description="The Seller's electronic address, to which the application-level response to the e-invoice file might be delivered. ", alias="endpointId")
    endpoint_scheme_id: Optional[StrictStr] = Field(default=None, description="The identification scheme identifier of the Seller’s electronic address. ", alias="endpointSchemeId")
    address_line1: Optional[StrictStr] = Field(default=None, description="The first line of the Seller’s address, which is often a street address or business name. ", alias="addressLine1")
    address_line2: Optional[StrictStr] = Field(default=None, description="The second line of the Seller’s address, which is often the name of a building. ", alias="addressLine2")
    postal_code: Optional[StrictStr] = Field(default=None, description="The short code that can identify the business address. ", alias="postalCode")
    city: Optional[StrictStr] = Field(default=None, description="The the name of the city where the business is located. ")
    state: Optional[StrictStr] = Field(default=None, description="The name of the state or province where the business is located. ")
    contact_name: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The name of the Seller contact to receive e-invoicing data. ", alias="contactName")
    email: Optional[StrictStr] = Field(default=None, description="The email address of the Seller contact to receive e-invoicing data. ")
    phone_number: Optional[StrictStr] = Field(default=None, description="The business phone number of the Seller contact to receive e-invoicing data. ", alias="phoneNumber")
    service_provider_id: Optional[StrictStr] = Field(default=None, description="The unique ID of the e-invoicing service provider that is associated to the business region. ", alias="serviceProviderId")
    digital_signature_enable: Optional[StrictBool] = Field(default=None, description="It will control that whether the pdf should be signed by vendor. ", alias="digitalSignatureEnable")
    digital_signature_box_enable: Optional[StrictBool] = Field(default=None, description="It will control whether the dignature box will be shown on the pdf. ", alias="digitalSignatureBoxEnable")
    digital_signature_box_pos_x: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="It is the X-axis that the box will be shown. ", alias="digitalSignatureBoxPosX")
    digital_signature_box_pos_y: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="It is the Y-axis that the box will be shown. ", alias="digitalSignatureBoxPosY")
    response_mapping: Optional[Dict[str, Any]] = Field(default=None, description="The response mapping of the e-invoicing business region. ", alias="responseMapping")
    process_type: Optional[StrictStr] = Field(default=None, description="The process type of the e-invoicing business region. ", alias="processType")
    invoice_enabled: Optional[StrictBool] = Field(default=None, description="It will control that whether the invoice should be supported by the process type or not. ", alias="invoiceEnabled")
    credit_note_enabled: Optional[StrictBool] = Field(default=None, description="It will control that whether the credit note should be supported by the process type or not. ", alias="creditNoteEnabled")
    debit_note_enabled: Optional[StrictBool] = Field(default=None, description="It will control that whether the debit note should be supported by the process type or not. ", alias="debitNoteEnabled")
    __properties: ClassVar[List[str]] = ["country", "businessName", "businessNumber", "businessNumberSchemaId", "tradeName", "taxRegisterNumber", "endpointId", "endpointSchemeId", "addressLine1", "addressLine2", "postalCode", "city", "state", "contactName", "email", "phoneNumber", "serviceProviderId", "digitalSignatureEnable", "digitalSignatureBoxEnable", "digitalSignatureBoxPosX", "digitalSignatureBoxPosY", "responseMapping", "processType", "invoiceEnabled", "creditNoteEnabled", "debitNoteEnabled"]

    @field_validator('process_type')
    def process_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Clearance', 'Cacellation', 'PostAudit', 'PEPPOL']):
            raise ValueError("must be one of enum values ('Clearance', 'Cacellation', 'PostAudit', 'PEPPOL')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PostBusinessRegionsRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PostBusinessRegionsRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "country": obj.get("country"),
            "businessName": obj.get("businessName"),
            "businessNumber": obj.get("businessNumber"),
            "businessNumberSchemaId": obj.get("businessNumberSchemaId"),
            "tradeName": obj.get("tradeName"),
            "taxRegisterNumber": obj.get("taxRegisterNumber"),
            "endpointId": obj.get("endpointId"),
            "endpointSchemeId": obj.get("endpointSchemeId"),
            "addressLine1": obj.get("addressLine1"),
            "addressLine2": obj.get("addressLine2"),
            "postalCode": obj.get("postalCode"),
            "city": obj.get("city"),
            "state": obj.get("state"),
            "contactName": obj.get("contactName"),
            "email": obj.get("email"),
            "phoneNumber": obj.get("phoneNumber"),
            "serviceProviderId": obj.get("serviceProviderId"),
            "digitalSignatureEnable": obj.get("digitalSignatureEnable"),
            "digitalSignatureBoxEnable": obj.get("digitalSignatureBoxEnable"),
            "digitalSignatureBoxPosX": obj.get("digitalSignatureBoxPosX"),
            "digitalSignatureBoxPosY": obj.get("digitalSignatureBoxPosY"),
            "responseMapping": obj.get("responseMapping"),
            "processType": obj.get("processType"),
            "invoiceEnabled": obj.get("invoiceEnabled"),
            "creditNoteEnabled": obj.get("creditNoteEnabled"),
            "debitNoteEnabled": obj.get("debitNoteEnabled")
        })
        return _obj


