# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field
from typing import Any, ClassVar, Dict, List, Optional
from zuora_sdk.models.charge_model_data_override import ChargeModelDataOverride
from zuora_sdk.models.discount_pricing_update import DiscountPricingUpdate
from zuora_sdk.models.recurring_delivery_pricing_update import RecurringDeliveryPricingUpdate
from zuora_sdk.models.recurring_flat_fee_pricing_update import RecurringFlatFeePricingUpdate
from zuora_sdk.models.recurring_per_unit_pricing_update import RecurringPerUnitPricingUpdate
from zuora_sdk.models.recurring_tiered_pricing_update import RecurringTieredPricingUpdate
from zuora_sdk.models.recurring_volume_pricing_update import RecurringVolumePricingUpdate
from zuora_sdk.models.usage_flat_fee_pricing_update import UsageFlatFeePricingUpdate
from zuora_sdk.models.usage_overage_pricing_update import UsageOveragePricingUpdate
from zuora_sdk.models.usage_per_unit_pricing_update import UsagePerUnitPricingUpdate
from zuora_sdk.models.usage_tiered_pricing_update import UsageTieredPricingUpdate
from zuora_sdk.models.usage_tiered_with_overage_pricing_update import UsageTieredWithOveragePricingUpdate
from zuora_sdk.models.usage_volume_pricing_update import UsageVolumePricingUpdate
from typing import Optional, Set
from typing_extensions import Self

class PricingUpdate(BaseModel):
    """
    PricingUpdate
    """ # noqa: E501
    charge_model_data: Optional[ChargeModelDataOverride] = Field(default=None, description="Container for charge model configuration data.  **Note**: This field is only available if you have the High Water Mark, Pre-Rated Pricing, or Multi-Attribute Pricing charge models enabled. The High Water Mark and Pre-Rated Pricing charge models are available for customers with Enterprise and Nine editions by default. If you are a Growth customer, see [Zuora Editions](https://knowledgecenter.zuora.com/BB_Introducing_Z_Business/C_Zuora_Editions) for pricing information. ", alias="chargeModelData")
    discount: Optional[DiscountPricingUpdate] = Field(default=None, description="Pricing information about a discount charge. ")
    recurring_delivery: Optional[RecurringDeliveryPricingUpdate] = Field(default=None, description="Pricing information about a recurring charge that uses the \"delivery\" charge model. This field is only available if you have the Delivery Pricing charge model enabled.  **Note**: The Delivery Pricing charge model is in the **Early Adopter** phase. We are actively soliciting feedback from a small set of early adopters before releasing it as generally available. If you want to join this early adopter program, submit a request at <a href=\"http://support.zuora.com/\" target=\"_blank\">Zuora Global Support</a>. ", alias="recurringDelivery")
    recurring_flat_fee: Optional[RecurringFlatFeePricingUpdate] = Field(default=None, description="Pricing information about a recurring charge that uses the \"flat fee\" charge model. In this charge model, the charge has a fixed price. ", alias="recurringFlatFee")
    recurring_per_unit: Optional[RecurringPerUnitPricingUpdate] = Field(default=None, description="Pricing information about a recurring charge that uses the \"per unit\" charge model. In this charge model, the charge has a fixed price per unit purchased. ", alias="recurringPerUnit")
    recurring_tiered: Optional[RecurringTieredPricingUpdate] = Field(default=None, description="Pricing information about a recurring charge that uses the \"tiered pricing\" charge model. In this charge model, the charge has cumulative pricing tiers that become effective as units are purchased. ", alias="recurringTiered")
    recurring_volume: Optional[RecurringVolumePricingUpdate] = Field(default=None, description="Pricing information about a recurring charge that uses the \"volume pricing\" charge model. In this charge model, the charge has a variable price per unit, depending on how many units are purchased. ", alias="recurringVolume")
    usage_flat_fee: Optional[UsageFlatFeePricingUpdate] = Field(default=None, description="Pricing information about a usage charge that uses the \"flat fee\" charge model. In this charge model, the charge has a fixed price. ", alias="usageFlatFee")
    usage_overage: Optional[UsageOveragePricingUpdate] = Field(default=None, description="Pricing information about a usage charge that uses the \"overage\" charge model. In this charge model, the charge has an allowance of free units and a fixed price per additional unit consumed. ", alias="usageOverage")
    usage_per_unit: Optional[UsagePerUnitPricingUpdate] = Field(default=None, description="Pricing information about a usage charge that uses the \"per unit\" charge model. In this charge model, the charge has a fixed price per unit consumed. ", alias="usagePerUnit")
    usage_tiered: Optional[UsageTieredPricingUpdate] = Field(default=None, description="Pricing information about a usage charge that uses the \"tiered pricing\" charge model. In this charge model, the charge has cumulative pricing tiers that become effective as units are consumed. ", alias="usageTiered")
    usage_tiered_with_overage: Optional[UsageTieredWithOveragePricingUpdate] = Field(default=None, description="Pricing information about a usage charge that uses the \"tiered with overage\" charge model. In this charge model, the charge has cumulative pricing tiers that become effective as units are consumed. The charge also has a fixed price per unit consumed beyond the limit of the final tier. ", alias="usageTieredWithOverage")
    usage_volume: Optional[UsageVolumePricingUpdate] = Field(default=None, description="Pricing information about a usage charge that uses the \"volume pricing\" charge model. In this charge model, the charge has a variable price per unit, depending on how many units are consumed. ", alias="usageVolume")
    __properties: ClassVar[List[str]] = ["chargeModelData", "discount", "recurringDelivery", "recurringFlatFee", "recurringPerUnit", "recurringTiered", "recurringVolume", "usageFlatFee", "usageOverage", "usagePerUnit", "usageTiered", "usageTieredWithOverage", "usageVolume"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PricingUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of charge_model_data
        if self.charge_model_data:
            _dict['chargeModelData'] = self.charge_model_data.to_dict()
        # override the default output from pydantic by calling `to_dict()` of discount
        if self.discount:
            _dict['discount'] = self.discount.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recurring_delivery
        if self.recurring_delivery:
            _dict['recurringDelivery'] = self.recurring_delivery.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recurring_flat_fee
        if self.recurring_flat_fee:
            _dict['recurringFlatFee'] = self.recurring_flat_fee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recurring_per_unit
        if self.recurring_per_unit:
            _dict['recurringPerUnit'] = self.recurring_per_unit.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recurring_tiered
        if self.recurring_tiered:
            _dict['recurringTiered'] = self.recurring_tiered.to_dict()
        # override the default output from pydantic by calling `to_dict()` of recurring_volume
        if self.recurring_volume:
            _dict['recurringVolume'] = self.recurring_volume.to_dict()
        # override the default output from pydantic by calling `to_dict()` of usage_flat_fee
        if self.usage_flat_fee:
            _dict['usageFlatFee'] = self.usage_flat_fee.to_dict()
        # override the default output from pydantic by calling `to_dict()` of usage_overage
        if self.usage_overage:
            _dict['usageOverage'] = self.usage_overage.to_dict()
        # override the default output from pydantic by calling `to_dict()` of usage_per_unit
        if self.usage_per_unit:
            _dict['usagePerUnit'] = self.usage_per_unit.to_dict()
        # override the default output from pydantic by calling `to_dict()` of usage_tiered
        if self.usage_tiered:
            _dict['usageTiered'] = self.usage_tiered.to_dict()
        # override the default output from pydantic by calling `to_dict()` of usage_tiered_with_overage
        if self.usage_tiered_with_overage:
            _dict['usageTieredWithOverage'] = self.usage_tiered_with_overage.to_dict()
        # override the default output from pydantic by calling `to_dict()` of usage_volume
        if self.usage_volume:
            _dict['usageVolume'] = self.usage_volume.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PricingUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "chargeModelData": ChargeModelDataOverride.from_dict(obj["chargeModelData"]) if obj.get("chargeModelData") is not None else None,
            "discount": DiscountPricingUpdate.from_dict(obj["discount"]) if obj.get("discount") is not None else None,
            "recurringDelivery": RecurringDeliveryPricingUpdate.from_dict(obj["recurringDelivery"]) if obj.get("recurringDelivery") is not None else None,
            "recurringFlatFee": RecurringFlatFeePricingUpdate.from_dict(obj["recurringFlatFee"]) if obj.get("recurringFlatFee") is not None else None,
            "recurringPerUnit": RecurringPerUnitPricingUpdate.from_dict(obj["recurringPerUnit"]) if obj.get("recurringPerUnit") is not None else None,
            "recurringTiered": RecurringTieredPricingUpdate.from_dict(obj["recurringTiered"]) if obj.get("recurringTiered") is not None else None,
            "recurringVolume": RecurringVolumePricingUpdate.from_dict(obj["recurringVolume"]) if obj.get("recurringVolume") is not None else None,
            "usageFlatFee": UsageFlatFeePricingUpdate.from_dict(obj["usageFlatFee"]) if obj.get("usageFlatFee") is not None else None,
            "usageOverage": UsageOveragePricingUpdate.from_dict(obj["usageOverage"]) if obj.get("usageOverage") is not None else None,
            "usagePerUnit": UsagePerUnitPricingUpdate.from_dict(obj["usagePerUnit"]) if obj.get("usagePerUnit") is not None else None,
            "usageTiered": UsageTieredPricingUpdate.from_dict(obj["usageTiered"]) if obj.get("usageTiered") is not None else None,
            "usageTieredWithOverage": UsageTieredWithOveragePricingUpdate.from_dict(obj["usageTieredWithOverage"]) if obj.get("usageTieredWithOverage") is not None else None,
            "usageVolume": UsageVolumePricingUpdate.from_dict(obj["usageVolume"]) if obj.get("usageVolume") is not None else None
        })
        return _obj


