# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from zuora_sdk.models.put_public_notification_definition_request_callout import PutPublicNotificationDefinitionRequestCallout
from zuora_sdk.models.put_public_notification_definition_request_filter_rule import PutPublicNotificationDefinitionRequestFilterRule
from typing import Optional, Set
from typing_extensions import Self

class PutPublicNotificationDefinitionRequest(BaseModel):
    """
    PutPublicNotificationDefinitionRequest
    """ # noqa: E501
    active: Optional[StrictBool] = Field(default=True, description="The status of the notification definition. The default value is `true`.")
    associated_account: Optional[StrictStr] = Field(default=None, description="Indicates with which type of account this notification is associated. Depending on your environment, you can use one of the following values: * `Account.Id`: ID of the primary customer account related to the notification. It is also the default value. * `ParentAccount.Id`: this option is available only if you have <a href=\"https://knowledgecenter.zuora.com/Billing/Subscriptions/Customer_Accounts/A_Customer_Account_Introduction#Customer_Hierarchy\" target=\"_blank\">Customer Hierarchy</a> enabled for your tenant. * `SubscriptionOwnerAccount.Id`: this option is available if the base object of the notification is Order Action.  **Note:** before specifying this field, we recommend that you use [Data Source](https://knowledgecenter.zuora.com/Billing/Reporting/D_Data_Sources_and_Exports/C_Data_Source_Reference) to check the available types of accounts for the current notification.   ", alias="associatedAccount")
    callout: Optional[PutPublicNotificationDefinitionRequestCallout] = None
    callout_active: Optional[StrictBool] = Field(default=False, description="The status of the callout action. The default value is `false`.", alias="calloutActive")
    communication_profile_id: Optional[StrictStr] = Field(default=None, description="The profile that notification definition belongs to. If you want to update the notification to a system notification, you should pass 'SystemNotification'. ' ", alias="communicationProfileId")
    description: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The description of the notification definition.")
    email_active: Optional[StrictBool] = Field(default=False, description="The status of the email action. The default value is `false`.", alias="emailActive")
    email_template_id: Optional[StrictStr] = Field(default=None, description="The ID of the email template. If emailActive is updated from false to true, an email template is required, and the EventType of the email template MUST be the same as the EventType of the notification definition. ", alias="emailTemplateId")
    filter_rule: Optional[PutPublicNotificationDefinitionRequestFilterRule] = Field(default=None, alias="filterRule")
    filter_rule_params: Optional[Dict[str, StrictStr]] = Field(default=None, description="The parameter values used to configure the filter rule. ", alias="filterRuleParams")
    name: Optional[Annotated[str, Field(strict=True, max_length=255)]] = Field(default=None, description="The name of the notification definition, which is unique in the profile.")
    __properties: ClassVar[List[str]] = ["active", "associatedAccount", "callout", "calloutActive", "communicationProfileId", "description", "emailActive", "emailTemplateId", "filterRule", "filterRuleParams", "name"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PutPublicNotificationDefinitionRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of callout
        if self.callout:
            _dict['callout'] = self.callout.to_dict()
        # override the default output from pydantic by calling `to_dict()` of filter_rule
        if self.filter_rule:
            _dict['filterRule'] = self.filter_rule.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PutPublicNotificationDefinitionRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "active": obj.get("active") if obj.get("active") is not None else True,
            "associatedAccount": obj.get("associatedAccount"),
            "callout": PutPublicNotificationDefinitionRequestCallout.from_dict(obj["callout"]) if obj.get("callout") is not None else None,
            "calloutActive": obj.get("calloutActive") if obj.get("calloutActive") is not None else False,
            "communicationProfileId": obj.get("communicationProfileId"),
            "description": obj.get("description"),
            "emailActive": obj.get("emailActive") if obj.get("emailActive") is not None else False,
            "emailTemplateId": obj.get("emailTemplateId"),
            "filterRule": PutPublicNotificationDefinitionRequestFilterRule.from_dict(obj["filterRule"]) if obj.get("filterRule") is not None else None,
            "filterRuleParams": obj.get("filterRuleParams"),
            "name": obj.get("name")
        })
        return _obj


