# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2023-10-24
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from zuora_sdk.models.credit_memo_item_from_invoice_item_finance_information import CreditMemoItemFromInvoiceItemFinanceInformation
from zuora_sdk.models.update_credit_memo_tax_item_request import UpdateCreditMemoTaxItemRequest
from typing import Optional, Set
from typing_extensions import Self

class UpdateCreditMemoItem(BaseModel):
    """
    UpdateCreditMemoItem
    """ # noqa: E501
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of the credit memo item. For tax-inclusive credit memo items, the amount indicates the credit memo item amount including tax. For tax-exclusive credit memo items, the amount indicates the credit memo item amount excluding tax")
    comment: Optional[StrictStr] = Field(default=None, description="Comments about the credit memo item.")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(default=None, description="The flag to exclude the credit memo item from revenue accounting.  **Note**: This field is only available if you have the Billing - Revenue Integration feature enabled.  ", alias="excludeItemBillingFromRevenueAccounting")
    finance_information: Optional[CreditMemoItemFromInvoiceItemFinanceInformation] = Field(default=None, alias="financeInformation")
    id: Annotated[str, Field(min_length=32, strict=True, max_length=32)] = Field(description="The ID of the credit memo item.")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The number of units for the credit memo item.")
    service_end_date: Optional[date] = Field(default=None, description="The service end date of the credit memo item.", alias="serviceEndDate")
    service_start_date: Optional[date] = Field(default=None, description="The service start date of the credit memo item.", alias="serviceStartDate")
    sku_name: Optional[StrictStr] = Field(default=None, description="The name of the SKU.", alias="skuName")
    tax_items: Optional[List[UpdateCreditMemoTaxItemRequest]] = Field(default=None, description="Container for credit memo taxation items.", alias="taxItems")
    unit_of_measure: Optional[StrictStr] = Field(default=None, description="The definable unit that you measure when determining charges.", alias="unitOfMeasure")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["amount", "comment", "excludeItemBillingFromRevenueAccounting", "financeInformation", "id", "quantity", "serviceEndDate", "serviceStartDate", "skuName", "taxItems", "unitOfMeasure"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UpdateCreditMemoItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of finance_information
        if self.finance_information:
            _dict['financeInformation'] = self.finance_information.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tax_items (list)
        _items = []
        if self.tax_items:
            for _item in self.tax_items:
                if _item:
                    _items.append(_item.to_dict())
            _dict['taxItems'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UpdateCreditMemoItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "amount": obj.get("amount"),
            "comment": obj.get("comment"),
            "excludeItemBillingFromRevenueAccounting": obj.get("excludeItemBillingFromRevenueAccounting"),
            "financeInformation": CreditMemoItemFromInvoiceItemFinanceInformation.from_dict(obj["financeInformation"]) if obj.get("financeInformation") is not None else None,
            "id": obj.get("id"),
            "quantity": obj.get("quantity"),
            "serviceEndDate": obj.get("serviceEndDate"),
            "serviceStartDate": obj.get("serviceStartDate"),
            "skuName": obj.get("skuName"),
            "taxItems": [UpdateCreditMemoTaxItemRequest.from_dict(_item) for _item in obj["taxItems"]] if obj.get("taxItems") is not None else None,
            "unitOfMeasure": obj.get("unitOfMeasure")
        })
        # store additional fields in additional_properties
        for _key in obj.keys():
            if _key not in cls.__properties:
                _obj.additional_properties[_key] = obj.get(_key)

        return _obj


